@@ -98,4 +98,4 @@ optimization_package='coopr' #'pulp'
 
 optimization_solver='gurobi'
 available_solvers = ['glpk','gurobi']#['glpk','gurobi','cplex']
-logging_level= logging.DEBUG
\ No newline at end of file
+logging_level= logging.INFO
\ No newline at end of file
