@@ -6,7 +6,7 @@ and  :class:`~powersystems.Line`. Each of these objects inherits
 an optimization framework from :class:`~optimization.OptimizationObject`.
 """
 
-from optimization import value,dual,sum_vars,OptimizationObject
+from optimization import value,dual,OptimizationObject
 from commonscripts import hours,drop_case_spaces,flatten,getattrL,unique,update_attributes,show_clock
 import config, bidding
 from schedule import FixedSchedule
@@ -148,12 +148,7 @@ class Generator(OptimizationObject):
         return self.bid(time).output_incremental(self.power(time)) if value(self.status(time)) else None
     def bid(self,time):
         ''':class:`~bidding.Bid` object for time'''
-        try: return self.get_component('bid', time)
-        except KeyError:
-            print str(time)
-            print self.children
-            print [str(c) for c in self.children]
-            raise 
+        return self.get_component('bid', time) 
     def getstatus(self,t,times): return dict(u=value(self.status(t)),P=value(self.power(t)),hoursinstatus=self.gethrsinstatus(t,times))
     def plot_cost_curve(self,P=None,filename=None): self.cost_model.plot(P,filename)
     def gethrsinstatus(self,tm,times):
@@ -161,9 +156,9 @@ class Generator(OptimizationObject):
         timesClipped=times[:times.index(tm)]
         try: 
             t_lastchange=(t for t in reversed(timesClipped) if value(self.status(t))!=status ).next()
-            return hours(tm-t_lastchange)
+            return hours(tm.End-t_lastchange.Start)
         except StopIteration: #no changes over whole time period
-            h=hours(tm-times[0])
+            h=hours(tm.End-times[0].Start)
             if value(self.status(times.initialTime)) == status: h+=self.initialStatusHours
             return h
     
@@ -196,8 +191,10 @@ class Generator(OptimizationObject):
                 if not self.mustrun: self.add_variable('status', 'u', time, kind='Binary')
                 else: self.add_variable('status', 'u', time, fixed_value=True)
                 self.add_variable('capacity', 'Pmax', time, low=0,high=self.Pmax)
-                self.add_variable('startupcost','Csu',time, low=0,high=self.startupcost)
-                self.add_variable('shutdowncost','Csd',time, low=0,high=self.shutdowncost)
+                if self.startupcost!=0: self.add_variable('startupcost','Csu',time, low=0,high=self.startupcost)
+                else: self.add_variable('startupcost','Csu',time, fixed_value=0)
+                if self.shutdowncost!=0: self.add_variable('shutdowncost','Csd',time, low=0,high=self.shutdowncost)
+                else: self.add_variable('shutdowncost','Csd',time, fixed_value=0)
             else: #ED or OPF problem, no commitments
                 self.add_variable('status', 'u', time,fixed_value=True)
                 self.add_variable('startupcost', 'Csu', time,fixed_value=0)
@@ -215,8 +212,7 @@ class Generator(OptimizationObject):
         #logging.debug('created {} variables {}'.format(str(self),show_clock()))
         return self.all_variables(times)
     def create_objective(self,times):
-        self.objective=sum_vars(self.cost(time) for time in times)
-        return self.objective
+        return sum(self.cost(time) for time in times)
 
     def create_constraints(self,times):
         '''create the optimization constraints for a generator over all times'''
@@ -239,9 +235,9 @@ class Generator(OptimizationObject):
             else: min_down_intervals_remaining_init=0
             #initial up down time
             if min_up_intervals_remaining_init>0: 
-                self.add_constraint('minuptime', tInitial, 0==sum_vars([(1-self.status(times[t])) for t in range(min_up_intervals_remaining_init)]))
+                self.add_constraint('minuptime', tInitial, 0==sum([(1-self.status(times[t])) for t in range(min_up_intervals_remaining_init)]))
             if min_down_intervals_remaining_init>0: 
-                self.add_constraint('mindowntime', tInitial, 0==sum_vars([self.status(times[t]) for t in range(min_down_intervals_remaining_init)]))
+                self.add_constraint('mindowntime', tInitial, 0==sum([self.status(times[t]) for t in range(min_down_intervals_remaining_init)]))
 
 
             #initial ramp rate
@@ -273,13 +269,13 @@ class Generator(OptimizationObject):
             if t >= min_up_intervals_remaining_init and self.minuptime>0:
                 no_shut_down=range(t,min(tEnd,t+min_up_intervals))
                 min_up_intervals_remaining=min(tEnd-t,min_up_intervals)
-                E = sum_vars([self.status(times[s]) for s in no_shut_down]) >= min_up_intervals_remaining*self.status_change(t,times)
+                E = sum([self.status(times[s]) for s in no_shut_down]) >= min_up_intervals_remaining*self.status_change(t,times)
                 self.add_constraint('min up time', time, E)
             #min down time        
             if t >= min_down_intervals_remaining_init and self.mindowntime>0:
                 no_start_up=range(t,min(tEnd,t+min_down_intervals))
                 min_down_intervals_remaining=min(tEnd-t,min_down_intervals)
-                E=sum_vars([1-self.status(times[s]) for s in no_start_up]) >= min_down_intervals_remaining * -1 * self.status_change(t,times)
+                E=sum([1-self.status(times[s]) for s in no_start_up]) >= min_down_intervals_remaining * -1 * self.status_change(t,times)
                 self.add_constraint('min down time', time, E)
                                         
             #ramping power
@@ -326,8 +322,8 @@ class Generator_nonControllable(Generator):
     def status(self,time): return True
     def set_initial_condition(self,time=None, P=None, u=None, hoursinstatus=None):
         try: 
-            if P is None: P=sorted(self.schedule.P.items())[0][1] #set initial value to first value
-            self.schedule.P[time]=P 
+            if P is None: P=sorted(self.schedule.energy.items())[0][1] #set initial value to first value
+            self.schedule.energy[time]=P 
         except AttributeError: pass #fixed schedule
     def getstatus(self,t,times): return {}
     def create_variables(self,times): return {}
@@ -360,15 +356,14 @@ class Load(OptimizationObject):
         update_attributes(self,locals()) #load in inputs
         self.init_optimization()
     def power(self,time): return self.get_variable('power',time) if self.shedding_allowed else self.schedule.get_energy(time)
-    def shed(self,time): return self.schedule.get_energy(time) - self.power(time) #power_to_energy(value(self.power(time)),time)
+    def shed(self,time): return self.schedule.get_energy(time) - self.power(time)
     def cost(self,time): return self.cost_shedding*self.shed(time) 
     def create_variables(self,times):
         if self.shedding_allowed:
             for time in times: self.add_variable('power','Pd',time,low=0,high=self.schedule.get_energy(time))
 
     def create_objective(self,times):
-        self.objective=sum_vars([ self.cost(time) for time in times])
-        return self.objective
+        return sum([ self.cost(time) for time in times])
 
     def __str__(self): return 'd{ind}'.format(ind=self.index)    
     def __int__(self): return self.index
@@ -448,12 +443,12 @@ class Bus(OptimizationObject):
     
     def angle(self,time): return self.get_variable('angle',time)
     def price(self,time): return dual(self.get_constraint('power balance',time))
-    def Pgen(self,t):   return sum_vars([gen.power(t) for gen in self.generators])
-    def Pload(self,t):  return sum_vars([ ld.power(t) for ld in self.loads])
+    def Pgen(self,t):   return sum(gen.power(t) for gen in self.generators)
+    def Pload(self,t):  return sum(ld.power(t) for ld in self.loads)
     def power_balance(self,t,Bmatrix,allBuses):
         if len(allBuses)==1: lineFlowsFromBus=0
-        else: lineFlowsFromBus=sum_vars([Bmatrix[self.index][otherBus.index]*otherBus.angle(t) for otherBus in allBuses]) #P_{ij}=sum_{i} B_{ij}*theta_j ???
-        return sum_vars([ -lineFlowsFromBus,-self.Pload(t),self.Pgen(t) ])
+        else: lineFlowsFromBus=sum([Bmatrix[self.index][otherBus.index]*otherBus.angle(t) for otherBus in allBuses]) #P_{ij}=sum_{i} B_{ij}*theta_j ???
+        return sum([ -lineFlowsFromBus,-self.Pload(t),self.Pgen(t) ])
     def create_variables(self,times):
         self.add_components(self.generators,'generators')
         self.add_components(self.loads,'loads')
@@ -475,10 +470,8 @@ class Bus(OptimizationObject):
         logging.debug('created bus variables ... returning {}'.format(show_clock()))
         return self.all_variables(times)
     def create_objective(self,times):
-        self.objective= \
-            sum_vars(gen.create_objective(times) for gen in self.generators) + \
-            sum_vars(load.create_objective(times) for load in self.loads)
-        return self.objective
+        return sum(gen.create_objective(times) for gen in self.generators) + \
+            sum(load.create_objective(times) for load in self.loads)
     def create_constraints(self,times,Bmatrix,buses):
         for gen in self.generators: gen.create_constraints(times)
         for load in self.loads: load.create_constraints(times)
@@ -530,7 +523,9 @@ class PowerSystem(OptimizationObject):
         #add system mode parameters to relevant components
         self.set_load_shedding(load_shedding_allowed) #set load shedding
         for load in loads:
-                try: load.cost_model.num_breakpoints=num_breakpoints
+                try: 
+                    load.cost_model.num_breakpoints=num_breakpoints
+                    load.cost_model.do_segmentation()
                 except AttributeError: pass #load has no cost model   
         for gen in generators:
             gen.dispatch_decommit_allowed=dispatch_decommit_allowed
@@ -600,20 +595,14 @@ class PowerSystem(OptimizationObject):
         logging.debug('... created power system vars... returning... {}'.format(show_clock()))
         return self.all_variables(times)
     def create_objective(self,times):
-        self.objective=sum_vars(bus.create_objective(times) for bus in self.buses) + sum_vars(line.create_objective(times) for line in self.lines)
-        return self.objective
+        return sum(bus.create_objective(times) for bus in self.buses) + sum(line.create_objective(times) for line in self.lines)
     def create_constraints(self,times):
         for bus in self.buses: bus.create_constraints(times,self.Bmatrix,self.buses)
         for line in self.lines: line.create_constraints(times,self.buses)
         #a system reserve constraint would go here
         return self.all_constraints(times)
-    # def clear_constraints(self):
-    #     self.constraints={}
-    #     for bus in self.buses: bus.clear_constraints()
-    #     for line in self.lines: line.clear_constraints()
     
     
 #def power_to_energy(P,time):
 #    return P*time.intervalhrs
-#def _call_generator_create_variables(gen,times): return gen.create_variables(times)
 
